/*
 *  Initiative - A role playing utility to track turns
 *  Copyright (C) 2002 Devon D Jones
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  The author of this program grants you the ability to use this code
 *  in conjunction with code that is covered under the Open Gaming License
 *
 * AboutBox.java
 *
 * Created on September 18, 2002, 5:38 PM
 */

package org.evilsoft.gaming.RPGeneration;

/**
 *
 * @author  devon
 */
public class AboutBox extends javax.swing.JDialog {
    
    /** Creates new form AboutBox */
    public AboutBox(java.awt.Frame parent, boolean modal, String version) {
        super(parent, modal);
        initComponents();
        labelVersion.setText("V. " + version);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jLabel3 = new javax.swing.JLabel();
        labelVersion = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel1 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel14 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jButton1 = new javax.swing.JButton();

        getContentPane().setLayout(new javax.swing.BoxLayout(getContentPane(), javax.swing.BoxLayout.Y_AXIS));

        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                closeDialog(evt);
            }
        });

        jLabel3.setFont(new java.awt.Font("Dialog", 1, 36));
        jLabel3.setForeground(new java.awt.Color(0, 51, 51));
        jLabel3.setText("RPGeneration");
        getContentPane().add(jLabel3);

        labelVersion.setText("V. 0.9.1");
        getContentPane().add(labelVersion);

        jLabel2.setText("by Soulcatcher (Devon Jones)");
        getContentPane().add(jLabel2);

        jLabel1.setText("soulcatcher@evilsoft.org");
        getContentPane().add(jLabel1);

        jLabel4.setText(" ");
        getContentPane().add(jLabel4);

        jLabel5.setText("Licensed under the GNU GPL");
        getContentPane().add(jLabel5);

        jLabel9.setText("Some data files are licensed under the 'Open Gaming License'");
        getContentPane().add(jLabel9);

        jLabel14.setText("Please reference the files OGL.txt & GPL.txt for more information");
        getContentPane().add(jLabel14);

        jLabel12.setText(" ");
        getContentPane().add(jLabel12);

        jLabel6.setText("Download From:");
        getContentPane().add(jLabel6);

        jLabel7.setText("http://www.evilsoft.org/Software");
        getContentPane().add(jLabel7);

        jLabel10.setText(" ");
        getContentPane().add(jLabel10);

        jLabel8.setText(" ");
        getContentPane().add(jLabel8);

        jLabel11.setText(" ");
        getContentPane().add(jLabel11);

        jButton1.setText("Done");
        jButton1.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButton1ActionPerformed(evt);
            }
        });

        getContentPane().add(jButton1);

        pack();
    }//GEN-END:initComponents

    private void jButton1ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButton1ActionPerformed
        setVisible(false);
        dispose();
    }//GEN-LAST:event_jButton1ActionPerformed
    
    /** Closes the dialog */
    private void closeDialog(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_closeDialog
        setVisible(false);
        dispose();
    }//GEN-LAST:event_closeDialog
    
   
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel labelVersion;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JButton jButton1;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel5;
    // End of variables declaration//GEN-END:variables
    
}
